@@ -6,6 +6,7 @@ from PySide6.QtWidgets import QApplication, QComboBox
 class Setup:
     DB_PATH = "jal.sqlite"
     DB_CONNECTION = "JAL.DB"
+    DB_REQUIRED_VERSION = 42
     SQLITE_MIN_VERSION = "3.35"
     MAIN_WND_NAME = "JAL_MainWindow"
     INIT_SCRIPT_PATH = 'jal_init.sql'
@@ -18,8 +19,9 @@ class Setup:
     REPORT_PATH = "reports"
     STATEMENT_PATH = "broker_statements"
     TEMPLATE_PATH = "templates"
+    TAX_REPORT_PATH = "tax_reports"
+    TAX_TREATY_PARAM = "tax_treaty"
     UPDATE_PREFIX = 'jal_delta_'
-    TARGET_SCHEMA = 38
     DEFAULT_ACCOUNT_PRECISION = 2
 
 
@@ -124,7 +126,7 @@ class PredefinedPeer:
 
 class MarketDataFeed:
     NA = -1
-    CBR = 0
+    FX = 0
     RU = 1
     US = 2
     EU = 3
