@@ -2,14 +2,13 @@ from jal.constants import Setup
 import sqlite3
 import logging
 import os
-import shutil
 from dateutil import tz
 from datetime import datetime
 from tempfile import TemporaryDirectory
 import tarfile
 
 from PySide6.QtWidgets import QApplication, QFileDialog, QMessageBox
-from jal.db.helpers import db_connection
+from jal.db.db import JalDB
 
 
 # ------------------------------------------------------------------------------
@@ -73,9 +72,23 @@ class JalBackup:
     def do_restore(self):
         with TemporaryDirectory(prefix=self.tmp_prefix) as tmp_path:
             with tarfile.open(self.backup_name, "r:gz") as tar:
-                tar.extractall(tmp_path)
+                def is_within_directory(directory, target):
+                    
+                    abs_directory = os.path.abspath(directory)
+                    abs_target = os.path.abspath(target)
+                    prefix = os.path.commonprefix([abs_directory, abs_target])
+                    return prefix == abs_directory
+                
+                def safe_extract(tar, path=".", members=None, *, numeric_owner=False):
+                    for member in tar.getmembers():
+                        member_path = os.path.join(path, member.name)
+                        if not is_within_directory(path, member_path):
+                            raise Exception("Attempted Path Traversal in Tar File")
+                    tar.extractall(path, members, numeric_owner=numeric_owner)
+                
+                safe_extract(tar, tmp_path)
             try:
-                shutil.move(tmp_path + os.sep + Setup.DB_PATH, self.file)
+                os.rename(tmp_path + os.sep + Setup.DB_PATH, self.file)
             except:
                 logging.warning(self.tr("Failed to restore backup file"))
                 return False
@@ -106,7 +119,7 @@ class JalBackup:
         self.get_filename(False)
         if self.backup_name is None:
             return
-        db_connection().close()
+        JalDB.connection().close()
 
         if not self.validate_backup():
             logging.error(self.tr("Wrong format of backup file"))
