@@ -6,7 +6,7 @@ from PySide6.QtWidgets import QHeaderView
 from jal.constants import CustomColor, PredefindedAccountType
 from jal.db.asset import JalAsset
 from jal.db.account import JalAccount
-from jal.db.settings import JalSettings
+from jal.widgets.delegates import FloatDelegate
 
 
 class BalancesModel(QAbstractTableModel):
@@ -19,6 +19,7 @@ class BalancesModel(QAbstractTableModel):
         self._active_only = True
         self._date = QDate.currentDate().endOfDay(Qt.UTC).toSecsSinceEpoch()
         self._columns = [self.tr("Account"), self.tr("Balance"), " ", self.tr("Balance, ")]
+        self._float_delegate = None
 
     def rowCount(self, parent=None):
         return len(self._data)
@@ -55,12 +56,12 @@ class BalancesModel(QAbstractTableModel):
                 return self._data[row]['account_name']
             else:
                 return PredefindedAccountType().get_name(self._data[row]['account_type'], default=self.tr("Total"))
-        elif column == 1:  # FIXME - should floating formatting be done by FloatDelegate? for columns 1 & 3
-            return f"{self._data[row]['balance']:,.2f}" if self._data[row]['balance'] != 0 else ''
+        elif column == 1:
+            return self._data[row]['balance']
         elif column == 2:
             return self._data[row]['currency_name'] if self._data[row]['balance'] != 0 else ''
         elif column == 3:
-            return f"{self._data[row]['balance_a']:,.2f}" if self._data[row]['balance_a'] != 0 else ''
+            return self._data[row]['balance_a']
         else:
             assert False
 
@@ -89,6 +90,9 @@ class BalancesModel(QAbstractTableModel):
         font = self._view.horizontalHeader().font()
         font.setBold(True)
         self._view.horizontalHeader().setFont(font)
+        self._float_delegate = FloatDelegate(2, allow_tail=False, empty_zero=True, parent=self._view)
+        self._view.setItemDelegateForColumn(1, self._float_delegate)
+        self._view.setItemDelegateForColumn(3, self._float_delegate)
 
     @Slot()
     def setCurrency(self, currency_id):
@@ -124,9 +128,7 @@ class BalancesModel(QAbstractTableModel):
         for account in accounts:
             value = value_adjusted = Decimal('0')
             assets = account.assets_list(self._date)
-            # Calculate cross-rate between account currency and display currency
-            rate = JalAsset(account.currency()).quote(self._date, JalSettings().getValue('BaseCurrency'))[1] / \
-                   JalAsset(self._currency).quote(self._date, JalSettings().getValue('BaseCurrency'))[1]
+            rate = JalAsset(account.currency()).quote(self._date, self._currency)[1]
             for asset_data in assets:
                 asset = asset_data['asset']
                 asset_value = asset_data['amount'] * asset.quote(self._date, account.currency())[1]
@@ -147,7 +149,7 @@ class BalancesModel(QAbstractTableModel):
                     "unreconciled": (account.last_operation_date() - account.reconciled_at())/86400,
                     "active": account.is_active()
                 })
-        balances = sorted(balances, key=lambda x: (x['account_type']))
+        balances = sorted(balances, key=lambda x: (x['account_type'], x['account_name']))
         self._data = []
         field_names = ["account_type", "account", "account_name", "currency", "currency_name", "balance", "balance_a",
                        "unreconciled", "active", "level"]
