@@ -1,10 +1,12 @@
 from typing import Union
 import os
+import sys
+import re
 import logging
 import sqlparse
 from pkg_resources import parse_version
 from PySide6.QtWidgets import QApplication, QMessageBox
-from PySide6.QtSql import QSql, QSqlDatabase, QSqlQuery
+from PySide6.QtSql import QSql, QSqlDatabase, QSqlQuery, QSqlTableModel
 
 from jal.constants import Setup
 from jal.db.helpers import get_dbfilename
@@ -39,12 +41,51 @@ class JalDBError:
 
 
 # ----------------------------------------------------------------------------------------------------------------------
-# FIXME all database calls should be via JalDB (or mate/descendant) class. Get rid of SQL calls from other code
+class JalSqlError:
+    def __init__(self, msg):
+        self.messages = {
+            'JAL_SQL_MSG_0001': self.tr("Investment account should have associated broker assigned"),
+            'JAL_SQL_MSG_0002': self.tr("Can't delete predefined category"),
+            'JAL_SQL_MSG_0003': self.tr("Incorrect currency assignment for an asset")
+        }
+        self.translations = {
+            'FOREIGN KEY constraint failed': self.tr("Data are referenced in another place and can't be modified")
+        }
+        if msg[:4] == 'JAL_':
+            self._message = self.messages[msg[:16]]
+            self._custom = True
+        elif msg in self.translations:
+            self._message = self.translations[msg]
+            self._custom = True
+        else:
+            self._message = msg
+            self._custom = False
+
+    def tr(self, text):
+        return QApplication.translate("JalSQLerror", text)
+
+    def show(self):
+        if "pytest" in sys.modules:  # Throw exception if we are in test mode or handle it if we are live
+            raise RuntimeError(self._message)
+        QMessageBox().warning(None, self.tr("Database error"), self._message, QMessageBox.Ok)
+
+    def custom(self):
+        return self._custom
+
+    def message(self):
+        return self._message
+
+
+# ----------------------------------------------------------------------------------------------------------------------
 class JalDB:
     _tables = []
+    _instances_with_cache = []
 
-    def __init__(self):
-        pass
+    # By default, db objects don't cache data. But if and object may cache db data we need to track it so parameter
+    # 'cached' to be set to True. Such objects should implement invalidate_cache(), class_cache() methods also.
+    def __init__(self, cached=None):
+        if cached:
+            self._instances_with_cache.append(self)
 
     def tr(self, text):
         return QApplication.translate("JalDB", text)
@@ -73,13 +114,14 @@ class JalDB:
             error = self.run_sql_script(db_path + Setup.INIT_SCRIPT_PATH)
             if error.code != JalDBError.NoError:
                 return error
-        schema_version = self.readSQL("SELECT value FROM settings WHERE name='SchemaVersion'")
-        if schema_version < Setup.TARGET_SCHEMA:
+        schema_version = self._read("SELECT value FROM settings WHERE name='SchemaVersion'")
+        if schema_version < Setup.DB_REQUIRED_VERSION:
             db.close()
             return JalDBError(JalDBError.OutdatedDbSchema)
-        elif schema_version > Setup.TARGET_SCHEMA:
+        elif schema_version > Setup.DB_REQUIRED_VERSION:
             db.close()
-            return JalDBError(JalDBError.NewerDbSchema)
+            return JalDBError(JalDBError.NewerDbSchema,
+                              details=f"(expected: {Setup.DB_REQUIRED_VERSION}, got: {schema_version})")
         self.enable_fk(True)
         self.enable_triggers(True)
 
@@ -88,7 +130,7 @@ class JalDB:
     # ------------------------------------------------------------------------------------------------------------------
     # Returns current version of sqlite library
     def get_engine_version(self):
-        return self.readSQL("SELECT sqlite_version()")
+        return self._read("SELECT sqlite_version()")
 
     # ------------------------------------------------------------------------------------------------------------------
     # This function returns SQLite connection used by JAL or fails with RuntimeError exception
@@ -102,41 +144,48 @@ class JalDB:
         return db
 
     # -------------------------------------------------------------------------------------------------------------------
-    # prepares SQL query from given sql_text
+    # Executes an SQL query from given sql_text
     # params is a list of tuples (":param", value) which are used to prepare SQL query
     # Current transaction will be committed if 'commit' set to true
     # Parameter 'forward_only' may be used for optimization
     # return value - QSqlQuery object (to allow iteration through result)
-    @staticmethod
-    def execSQL(sql_text, params=None, forward_only=True, commit=False):
+    @classmethod
+    def _exec(cls, sql_text, params=None, forward_only=True, commit=False):
         if params is None:
             params = []
-        db = JalDB.connection()
+        db = cls.connection()
         query = QSqlQuery(db)
         query.setForwardOnly(forward_only)
         if not query.prepare(sql_text):
             logging.error(f"SQL query preparation failure: '{query.lastError().text()}' for query '{sql_text}'")
             return None
+        query_params = set(re.findall(r":(\w+)", sql_text, re.IGNORECASE))  # get all parameter names in query text
+        assert len(query_params) == len(params), f"SQL: wrong number of parameters {params} for '{sql_text}'"
         for param in params:
             query.bindValue(param[0], param[1])
             assert query.boundValue(param[0]) == param[1], f"SQL: failed to assign parameter {param} in '{sql_text}'"
         if not query.exec():
-            logging.error(f"SQL failure: '{query.lastError().text()}' for query '{sql_text}' with params '{params}'")
+            error = JalSqlError(query.lastError().text())
+            if error.custom():
+                error.show()
+            else:
+                logging.error(f"SQL failure: '{error.message()}' for query '{sql_text}' with params '{params}'")
             return None
         if commit:
             db.commit()
         return query
 
     # ------------------------------------------------------------------------------------------------------------------
-    # Calls execSQL() method with given SQL query and parameters and returns its result or None if result is empty
+    # Reads the result of 'sql_test' query from the database (with given params - the same as for _exec() method)
+    # returns result of the query or None if result is empty
     # named = False: result is packed into a list of field values
     # named = True: result is packet into a dictionary with field names as keys
     # check_unique = True: checks that only 1 record was returned by query, otherwise returns None
-    @staticmethod
-    def readSQL(sql_text, params=None, named=False, check_unique=False):
-        query = JalDB.execSQL(sql_text, params)
+    @classmethod
+    def _read(cls, sql_text, params=None, named=False, check_unique=False):
+        query = cls._exec(sql_text, params)
         if query.next():
-            res = JalDB.readSQLrecord(query, named=named)
+            res = cls._read_record(query, named=named)
             if check_unique and query.next():
                 return None  # More than one record in result when only one expected
             return res
@@ -147,14 +196,19 @@ class JalDB:
     # Method takes current active record of given query and returns its values as:
     # named = False: a list of field values
     # named = True: a dictionary with field names as keys
-    @staticmethod
-    def readSQLrecord(query, named=False):
+    @classmethod
+    def _read_record(cls, query, named=False, cast=None):
+        if cast is None:
+            cast = []
         values = {} if named else []
+        if cast:
+            assert len(cast) == query.record().count()
         for i in range(query.record().count()):
+            value = cast[i](query.value(i)) if cast else query.value(i)
             if named:
-                values[query.record().fieldName(i)] = query.value(i)
+                values[query.record().fieldName(i)] = value
             else:
-                values.append(query.value(i))
+                values.append(value)
         if values:
             if len(values) == 1 and not named:
                 return values[0]
@@ -163,29 +217,44 @@ class JalDB:
         else:
             return None
 
+    # ------------------------------------------------------------------------------------------------------------------
+    def invalidate_cache(self):
+        cache_classes = set()   # a list of classes that were already invalidated and don't need extra action
+        for item in self._instances_with_cache:
+            if item.class_cache() and type(item) in cache_classes:
+                continue
+            else:
+                cache_classes.add(type(item))
+            item.invalidate_cache()
+
+    # Method returns true if data are cached on a class level, not in every instance
+    @classmethod
+    def class_cache(cls) -> True:
+        return False
+
     # ------------------------------------------------------------------------------------------------------------------
     # Enables DB triggers if enable == True and disables it otherwise
     def enable_triggers(self, enable):
         if enable:
-            _ = self.execSQL("UPDATE settings SET value=1 WHERE name='TriggersEnabled'", commit=True)
+            _ = self._exec("UPDATE settings SET value=1 WHERE name='TriggersEnabled'", commit=True)
         else:
-            _ = self.execSQL("UPDATE settings SET value=0 WHERE name='TriggersEnabled'", commit=True)
+            _ = self._exec("UPDATE settings SET value=0 WHERE name='TriggersEnabled'", commit=True)
 
     # ------------------------------------------------------------------------------------------------------------------
     # Set synchronous mode ON if synchronous == True and OFF it otherwise
     def set_synchronous(self, synchronous):
         if synchronous:
-            _ = self.execSQL("PRAGMA synchronous = ON")
+            _ = self._exec("PRAGMA synchronous = ON")
         else:
-            _ = self.execSQL("PRAGMA synchronous = OFF")
+            _ = self._exec("PRAGMA synchronous = OFF")
 
     # ------------------------------------------------------------------------------------------------------------------
     # Enables DB foreign keys if enable == True and disables it otherwise
     def enable_fk(self, enable):
         if enable:
-            _ = self.execSQL("PRAGMA foreign_keys = ON")
+            _ = self._exec("PRAGMA foreign_keys = ON")
         else:
-            _ = self.execSQL("PRAGMA foreign_keys = OFF")
+            _ = self._exec("PRAGMA foreign_keys = OFF")
 
     # Method loads sql script into database
     def run_sql_script(self, script_file) -> JalDBError:
@@ -194,8 +263,8 @@ class JalDB:
                 statements = sqlparse.split(sql_script)
                 for statement in statements:
                     clean_statement = sqlparse.format(statement, strip_comments=True)
-                    if self.execSQL(clean_statement, commit=False) is None:
-                        _ = self.execSQL("ROLLBACK")
+                    if self._exec(clean_statement, commit=False) is None:
+                        _ = self._exec("ROLLBACK")
                         self.connection().close()
                         return JalDBError(JalDBError.SQLFailure, f"FAILED: {clean_statement}")
                     else:
@@ -211,12 +280,12 @@ class JalDB:
                                  QMessageBox.Yes, QMessageBox.No) == QMessageBox.No:
             return JalDBError(JalDBError.OutdatedDbSchema)
         db = self.connection()
-        version = self.readSQL("SELECT value FROM settings WHERE name='SchemaVersion'")
+        version = self._read("SELECT value FROM settings WHERE name='SchemaVersion'")
         try:
             schema_version = int(version)
         except ValueError:
-            return JalDBError(JalDBError.DbInitFailure)
-        for step in range(schema_version, Setup.TARGET_SCHEMA):
+            return JalDBError(JalDBError.DbInitFailure, details=f"(db schema: {version}")
+        for step in range(schema_version, Setup.DB_REQUIRED_VERSION):
             delta_file = db_path + Setup.UPDATES_PATH + os.sep + Setup.UPDATE_PREFIX + f"{step + 1}.sql"
             logging.info(f"Applying delta schema {step}->{step + 1} from {delta_file}")
             error = self.run_sql_script(delta_file)
@@ -276,7 +345,7 @@ class JalDB:
                 query_text += f"{field} = :{field} AND "
                 params.append((f":{field}", data[field]))
         query_text = query_text[:-len(" AND ")]   # cut extra tail
-        oid = self.readSQL(query_text, params)
+        oid = self._read(query_text, params)
         if oid:
             return int(oid)
         return 0
@@ -295,17 +364,27 @@ class JalDB:
                 values_text += f":{field}, "
                 params.append((f":{field}", data[field]))
         query_text = query_text[:-2] + ") " + values_text[:-2] + ")"
-        query = self.execSQL(query_text, params, commit=True)
+        query = self._exec(query_text, params, commit=True)
         return query.lastInsertId()
 
-    # Returns value of 'field_name' from 'table_name' where 'key_field' is equal to 'search_value'
-    @staticmethod
-    def get_db_value(table_name: str, field_name: str, key_field: str, search_value: Union[int, str]) -> str:
-        if table_name not in JalDB._tables:
-            return ''
+
+# -------------------------------------------------------------------------------------------------------------------
+# Subclassing to hide db connection details
+class JalModel(QSqlTableModel):
+    def __init__(self, parent, table_name):
+        super().__init__(parent=parent, db=JalDB.connection())
+        self.setTable(table_name)
+        self._table = table_name
+
+    # Returns value of 'field_name' where 'key_field' is equal to 'search_value'
+    def get_value(self, field_name: str, key_field: str, search_value: Union[int, str]) -> str:
         if ' ' in field_name or ' ' in key_field:
             return ''
         if type(search_value) == str:
             search_value = "'" + search_value + "'"   # Enclose string into quotes
-        query_text = f"SELECT {field_name} FROM {table_name} WHERE {key_field}={search_value}"
-        return JalDB.readSQL(query_text)
+        self.setFilter(f"{key_field}={search_value}")
+        self.select()
+        result = self.record(0).field(field_name).value()
+        self.setFilter('')
+        self.select()
+        return result
