@@ -1,8 +1,9 @@
 import os
+from datetime import datetime, timezone, timedelta
+from decimal import Decimal
 from PySide6.QtCore import QLocale
 from PySide6.QtGui import QIcon
 from jal.constants import Setup
-from decimal import Decimal
 
 
 # -------------------------------------------------------------------------------------------------------------------
@@ -66,3 +67,17 @@ def get_dbfilename(app_path):
 # returns QIcon loaded from the file with icon_name located in folder Setup.ICONS_PATH
 def load_icon(icon_name) -> QIcon:
     return QIcon(get_app_path() + Setup.ICONS_PATH + os.sep + icon_name)
+
+# -------------------------------------------------------------------------------------------------------------------
+# Returns timestamp of the first second of the year of given timestamp
+def year_begin(timestamp: int) -> int:
+    begin = datetime.utcfromtimestamp(timestamp).replace(month=1, day=1, hour=0, minute=0, second=0)
+    return int(begin.replace(tzinfo=timezone.utc).timestamp())
+
+# Returns timestamp of the last second of the year of given timestamp
+def year_end(timestamp: int) -> int:
+    end = datetime.utcfromtimestamp(timestamp).replace(month=1, day=1, hour=0, minute=0, second=0)
+    end = end.replace(year=end.year + 1) - timedelta(seconds=1)
+    return int(end.replace(tzinfo=timezone.utc).timestamp())
+
+# -------------------------------------------------------------------------------------------------------------------
\ No newline at end of file
