@@ -3,12 +3,11 @@ from PySide6.QtCore import Qt, QAbstractItemModel, QModelIndex
 from PySide6.QtSql import QSqlTableModel, QSqlRelationalTableModel
 from PySide6.QtGui import QFont
 from PySide6.QtWidgets import QHeaderView, QMessageBox
-from jal.db.db import JalDB
-from jal.widgets.helpers import decodeError
+from jal.db.db import JalDB, JalSqlError
 
 
 # ----------------------------------------------------------------------------------------------------------------------
-class AbstractReferenceListModel(QSqlRelationalTableModel):
+class AbstractReferenceListModel(QSqlRelationalTableModel, JalDB):
     @property
     def completion_model(self):
         return self._completion_model
@@ -26,13 +25,13 @@ class AbstractReferenceListModel(QSqlRelationalTableModel):
         self._hidden = []
         self._stretch = None
         self._default_values = {}   # To fill in default values for fields allowed to be NULL
-        QSqlRelationalTableModel.__init__(self, parent=parent_view, db=JalDB.connection())
+        QSqlRelationalTableModel.__init__(self, parent=parent_view, db=self.connection())
         self.setJoinMode(QSqlRelationalTableModel.LeftJoin)
         self.setTable(self._table)
         self.setEditStrategy(QSqlTableModel.OnManualSubmit)
         self.select()
         # This is auxiliary 'plain' model of the same table - to be given as QCompleter source of data
-        self._completion_model = QSqlTableModel(parent=parent_view, db=JalDB.connection())
+        self._completion_model = QSqlTableModel(parent=parent_view, db=self.connection())
         self._completion_model.setTable(self._table)
         self._completion_model.select()
 
@@ -89,7 +88,7 @@ class AbstractReferenceListModel(QSqlRelationalTableModel):
         return self.getFieldValue(self.getId(index), self._default_name)
 
     def getFieldValue(self, item_id, field_name):
-        return JalDB.readSQL(f"SELECT {field_name} FROM {self._table} WHERE id=:id", [(":id", item_id)])
+        return self._read(f"SELECT {field_name} FROM {self._table} WHERE id=:id", [(":id", item_id)])
 
     def addElement(self, index, in_group=0):
         row = index.row() if index.isValid() else 0
@@ -117,21 +116,18 @@ class AbstractReferenceListModel(QSqlRelationalTableModel):
             self._deleted_rows = []
         else:
             error_code = self.lastError().nativeErrorCode()
-            if error_code == '1299':
-                prefix = "NOT NULL constraint failed: " + self.tableName() + "."
-                if self.lastError().databaseText().startswith(prefix):
-                    field_name = self.lastError().databaseText()[len(prefix):]
+            null_pfx = "NOT NULL constraint failed: " + self.tableName() + "."
+            if error_code == '1299' and self.lastError().databaseText().startswith(null_pfx):
+                    field_name = self.lastError().databaseText()[len(null_pfx):]
                     header_title = self.tableName() + ":" + self.headerData(self.fieldIndex(field_name))
                     QMessageBox().warning(self._view, self.tr("Data are incomplete"),
                                           self.tr("Column has no valid value: " + header_title), QMessageBox.Ok)
-                else:
-                    logging.fatal(self.tr("Submit failed: ") + decodeError(self.lastError().text()))
-            elif error_code == '1811':   # Foreign key constraint failed
-                QMessageBox().warning(self._view, self.tr("Data are in use"),
-                                      self.tr("Data are referenced in another place and can't be modified"),
-                                      QMessageBox.Ok)
             else:
-                logging.fatal(self.tr("Submit failed: ") + decodeError(self.lastError().text()))
+                error = JalSqlError(self.lastError().databaseText())
+                if error.custom():
+                    error.show()
+                else:
+                    logging.fatal(self.tr("Submit failed: ") + error.message())
         return result
 
     def revertAll(self):
@@ -139,21 +135,34 @@ class AbstractReferenceListModel(QSqlRelationalTableModel):
         super().revertAll()
 
     def locateItem(self, item_id, use_filter=''):
-        raise NotImplementedError(f"locateItem() method is not defined  in {type(self).__name__} class")
+        if use_filter:
+            use_filter = f"WHERE {use_filter}"
+        row = self._read(f"SELECT row_number FROM ("
+                         f"SELECT ROW_NUMBER() OVER (ORDER BY {self._default_name}) AS row_number, id "
+                         f"FROM {self._table} {use_filter}) WHERE id=:id", [(":id", item_id)])
+        if row is None:
+            return QModelIndex()
+        return self.index(row - 1, 0)
 
     def updateItemType(self, index, new_type):
         id = self.getId(index)
-        JalDB.execSQL(f"UPDATE {self._table} SET {self._group_by}=:new_type WHERE id=:id",
-                      [(":new_type", new_type), (":id", id)])
+        self._exec(f"UPDATE {self._table} SET {self._group_by}=:new_type WHERE id=:id",
+                   [(":new_type", new_type), (":id", id)])
 
     def filterBy(self, field_name, value):
         self._filter_by = field_name
         self._filter_value = value
         self.setFilter(f"{self._table}.{field_name} = {value}")
 
+    # returns group id for given item
+    def getGroupId(self, item_id: int) -> int:
+        group_id = self._read(f"SELECT {self._group_by} FROM {self._table} WHERE id=:id", [(":id", item_id)])
+        group_id = 0 if group_id is None else group_id
+        return group_id
+
 
 # ----------------------------------------------------------------------------------------------------------------------
-class SqlTreeModel(QAbstractItemModel):
+class SqlTreeModel(QAbstractItemModel, JalDB):
     ROOT_PID = 0
 
     @property
@@ -163,11 +172,13 @@ class SqlTreeModel(QAbstractItemModel):
     def __init__(self, table, parent_view):
         super().__init__(parent_view)
         self._table = table
+        self._columns = []
         self._view = parent_view
         self._default_name = "name"
         self._stretch = None
+        self._filter_text = ''
         # This is auxiliary 'plain' model of the same table - to be given as QCompleter source of data
-        self._completion_model = QSqlTableModel(parent=parent_view, db=JalDB.connection())
+        self._completion_model = QSqlTableModel(parent=parent_view, db=self.connection())
         self._completion_model.setTable(self._table)
         self._completion_model.select()
 
@@ -178,8 +189,12 @@ class SqlTreeModel(QAbstractItemModel):
             parent_id = self.ROOT_PID
         else:
             parent_id = parent.internalId()
-        child_id = JalDB.readSQL(f"SELECT id FROM {self._table} WHERE pid=:pid LIMIT 1 OFFSET :row_n",
-                                 [(":pid", parent_id), (":row_n", row)])
+        if not self._filter_text:
+            child_id = self._read(f"SELECT id FROM {self._table} WHERE pid=:pid LIMIT 1 OFFSET :row_n",
+                                  [(":pid", parent_id), (":row_n", row)])
+        else:  # display a plain list in a filter mode
+            child_id = self._read(f"SELECT id FROM {self._table} WHERE {self._filter_text} LIMIT 1 OFFSET :row_n",
+                                  [(":row_n", row)])
         if child_id:
             return self.createIndex(row, column, id=child_id)
         return QModelIndex()
@@ -188,13 +203,13 @@ class SqlTreeModel(QAbstractItemModel):
         if not index.isValid():
             return QModelIndex()
         child_id = index.internalId()
-        parent_id = JalDB.readSQL(f"SELECT pid FROM {self._table} WHERE id=:id", [(":id", child_id)])
+        parent_id = self._read(f"SELECT pid FROM {self._table} WHERE id=:id", [(":id", child_id)])
         if parent_id == self.ROOT_PID:
             return QModelIndex()
-        row = JalDB.readSQL(f"SELECT row_number FROM ("
-                             f"SELECT ROW_NUMBER() OVER (ORDER BY id) AS row_number, id, pid "
-                             f"FROM {self._table} WHERE pid IN (SELECT pid FROM {self._table} WHERE id=:id)) "
-                             f"WHERE id=:id", [(":id", parent_id)])
+        row = self._read(f"SELECT row_number FROM ("
+                         f"SELECT ROW_NUMBER() OVER (ORDER BY id) AS row_number, id, pid "
+                         f"FROM {self._table} WHERE pid IN (SELECT pid FROM {self._table} WHERE id=:id)) "
+                         f"WHERE id=:id", [(":id", parent_id)])
         return self.createIndex(row-1, 0, id=parent_id)
 
     def rowCount(self, parent=None):
@@ -202,7 +217,12 @@ class SqlTreeModel(QAbstractItemModel):
             parent_id = self.ROOT_PID
         else:
             parent_id = parent.internalId()
-        count = JalDB.readSQL(f"SELECT COUNT(id) FROM {self._table} WHERE pid=:pid", [(":pid", parent_id)])
+        if not self._filter_text:
+            count = self._read(f"SELECT COUNT(id) FROM {self._table} WHERE pid=:pid", [(":pid", parent_id)])
+        else:
+            if parent_id:  # no children if we are in filter mode and display a plain list
+                return 0
+            count = self._read(f"SELECT COUNT(id) FROM {self._table} WHERE {self._filter_text}")
         if count:
             return int(count)
         else:
@@ -223,8 +243,8 @@ class SqlTreeModel(QAbstractItemModel):
         if role == Qt.DisplayRole:
             col = index.column()
             if (col >= 0) and (col < len(self._columns)):
-                return JalDB.readSQL(f"SELECT {self._columns[col][0]} FROM {self._table} WHERE id=:id",
-                                     [(":id", item_id)])
+                return self._read(f"SELECT {self._columns[col][0]} FROM {self._table} WHERE id=:id",
+                                  [(":id", item_id)])
             else:
                 return None
         return None
@@ -236,9 +256,9 @@ class SqlTreeModel(QAbstractItemModel):
             return False
         item_id = index.internalId()
         col = index.column()
-        JalDB.connection().transaction()
-        _ = JalDB.execSQL(f"UPDATE {self._table} SET {self._columns[col][0]}=:value WHERE id=:id",
-                          [(":id", item_id), (":value", value)])
+        self.connection().transaction()
+        _ = self._exec(f"UPDATE {self._table} SET {self._columns[col][0]}=:value WHERE id=:id",
+                       [(":id", item_id), (":value", value)])
         self.dataChanged.emit(index, index, Qt.DisplayRole | Qt.EditRole)
         return True
 
@@ -271,18 +291,16 @@ class SqlTreeModel(QAbstractItemModel):
         return index.internalId()
 
     def getName(self, index):
-        item_id = self.getId(index)
-        if item_id is not None:
-            self.getFieldValue(item_id, self._default_name)
+        return self.getFieldValue(self.getId(index), self._default_name)
 
     def getFieldValue(self, item_id, field_name):
-        return JalDB.readSQL(f"SELECT {field_name} FROM {self._table} WHERE id=:id", [(":id", item_id)])
+        return self._read(f"SELECT {field_name} FROM {self._table} WHERE id=:id", [(":id", item_id)])
 
     def deleteWithChilderen(self, parent_id: int) -> None:
-        query = JalDB.execSQL(f"SELECT id FROM {self._table} WHERE pid=:pid", [(":pid", parent_id)])
+        query = self._exec(f"SELECT id FROM {self._table} WHERE pid=:pid", [(":pid", parent_id)])
         while query.next():
             self.deleteWithChilderen(query.value(0))
-        _ = JalDB.execSQL(f"DELETE FROM {self._table} WHERE id=:id", [(":id", parent_id)])
+        _ = self._exec(f"DELETE FROM {self._table} WHERE id=:id", [(":id", parent_id)])
 
     def insertRows(self, row, count, parent=None):
         if parent is None:
@@ -293,8 +311,8 @@ class SqlTreeModel(QAbstractItemModel):
             parent_id = parent.internalId()
 
         self.beginInsertRows(parent, row, row + count - 1)
-        JalDB.connection().transaction()
-        _ = JalDB.execSQL(f"INSERT INTO {self._table}(pid, name) VALUES (:pid, '')", [(":pid", parent_id)])
+        self.connection().transaction()
+        _ = self._exec(f"INSERT INTO {self._table}(pid, name) VALUES (:pid, '')", [(":pid", parent_id)])
         self.endInsertRows()
         self.layoutChanged.emit()
         return True
@@ -308,9 +326,9 @@ class SqlTreeModel(QAbstractItemModel):
             parent_id = parent.internalId()
 
         self.beginRemoveRows(parent, row, row + count - 1)
-        JalDB.connection().transaction()
-        query = JalDB.execSQL(f"SELECT id FROM {self._table} WHERE pid=:pid LIMIT :row_c OFFSET :row_n",
-                              [(":pid", parent_id), (":row_c", count), (":row_n", row)])
+        self.connection().transaction()
+        query = self._exec(f"SELECT id FROM {self._table} WHERE pid=:pid LIMIT :row_c OFFSET :row_n",
+                           [(":pid", parent_id), (":row_c", count), (":row_n", row)])
         while query.next():
             self.deleteWithChilderen(query.value(0))
         self.endRemoveRows()
@@ -330,12 +348,12 @@ class SqlTreeModel(QAbstractItemModel):
         assert self.removeRows(row, 1, index.parent())
 
     def submitAll(self):
-        _ = JalDB.execSQL("COMMIT")
+        _ = self._exec("COMMIT")
         self.layoutChanged.emit()
         return True
 
     def revertAll(self):
-        _ = JalDB.execSQL("ROLLBACK")
+        _ = self._exec("ROLLBACK")
         self.layoutChanged.emit()
 
     # expand all parent elements for tree element with given index
@@ -347,15 +365,16 @@ class SqlTreeModel(QAbstractItemModel):
 
     # find item by ID and make it selected in associated self._view
     def locateItem(self, item_id):
-        row = JalDB.readSQL(f"SELECT row_number FROM ("
-                             f"SELECT ROW_NUMBER() OVER (ORDER BY id) AS row_number, id, pid "
-                             f"FROM {self._table} WHERE pid IN (SELECT pid FROM {self._table} WHERE id=:id)) "
-                             f"WHERE id=:id", [(":id", item_id)])
+        row = self._read(f"SELECT row_number FROM ("
+                         f"SELECT ROW_NUMBER() OVER (ORDER BY id) AS row_number, id, pid "
+                         f"FROM {self._table} WHERE pid IN (SELECT pid FROM {self._table} WHERE id=:id)) "
+                         f"WHERE id=:id", [(":id", item_id)])
         if row is None:
             return
         item_idx = self.createIndex(row-1, 0, id=item_id)
         self.expand_parent(item_idx)
         self._view.setCurrentIndex(item_idx)
 
-    def setFilter(self, filter_str):
-        pass
\ No newline at end of file
+    def setFilter(self, text):
+        self._filter_text = text
+        self.layoutChanged.emit()
