@@ -3,9 +3,9 @@ from decimal import Decimal
 import pandas as pd
 from PySide6.QtCore import Qt, QAbstractTableModel, QDate
 from PySide6.QtGui import QFont
+from jal.constants import PredefinedAsset
 from jal.db.account import JalAccount
 from jal.db.asset import JalAsset
-from jal.db.settings import JalSettings
 from jal.ui.reports.ui_tax_estimation import Ui_TaxEstimationDialog
 from jal.widgets.mdi import MdiWidget
 from jal.widgets.helpers import ts2d
@@ -89,13 +89,13 @@ class TaxEstimator(MdiWidget, Ui_TaxEstimationDialog):
         self.ready = True
 
     def prepare_tax(self):
+        ruble_id = JalAsset(data={'symbol': 'RUB', 'type_id': PredefinedAsset.Money}, search=True, create=False).id()
         account = JalAccount(self.account_id)
         asset = JalAsset(self.asset_id)
         account_currency = JalAsset(account.currency())
         self.currency_name = account_currency.symbol()
         self.quote = asset.quote(QDate.currentDate().endOfDay(Qt.UTC).toSecsSinceEpoch(), account.currency())[1]
-        self.rate = account_currency.quote(QDate.currentDate().endOfDay(Qt.UTC).toSecsSinceEpoch(),
-                                           JalSettings().getValue('BaseCurrency'))[1]
+        self.rate = account_currency.quote(QDate.currentDate().endOfDay(Qt.UTC).toSecsSinceEpoch(), ruble_id)[1]
         positions = account.open_trades_list(asset)
         table = []
         profit = Decimal('0')
@@ -105,16 +105,15 @@ class TaxEstimator(MdiWidget, Ui_TaxEstimationDialog):
         for position in positions:
             qty = position['remaining_qty']
             price = position['price']
-            o_rate = account_currency.quote(position['operation'].settlement(),
-                                            JalSettings().getValue('BaseCurrency'))[1]
+            o_rate = account_currency.quote(position['operation'].settlement(), ruble_id)[1]
             position_profit = qty * (self.quote - price)
             position_profit_rub = qty * (self.quote * self.rate - price * o_rate)
             tax = Decimal('0.13') * position_profit_rub if position_profit_rub > Decimal('0') else Decimal('0')
             table.append({
                 'timestamp': ts2d(position['operation'].timestamp()),
                 'qty': qty,
-                'o_rate': o_rate,
                 'o_price': price,
+                'o_rate': o_rate,
                 'profit': position_profit,
                 'profit_rub': position_profit_rub,
                 'tax': tax
