@@ -12,9 +12,7 @@ from PySide6.QtWidgets import QApplication, QDialog, QListWidgetItem
 
 from jal.ui.ui_update_quotes_window import Ui_UpdateQuotesDlg
 from jal.constants import MarketDataFeed, PredefinedAsset
-from jal.db.db import JalDB
 from jal.db.asset import JalAsset
-from jal.db.settings import JalSettings
 from jal.net.helpers import get_web_data, post_web_data, isEnglish
 
 
@@ -66,9 +64,60 @@ class QuoteDownloader(QObject):
     def __init__(self):
         super().__init__()
         self.CBR_codes = None
-        self.data_loaders = {
+
+    def showQuoteDownloadDialog(self, parent):
+        dialog = QuotesUpdateDialog(parent)
+        if dialog.exec():
+            self.DownloadData(dialog.getStartDate(), dialog.getEndDate(), dialog.getSourceList())
+            self.download_completed.emit()
+
+    def DownloadData(self, start_timestamp, end_timestamp, sources_list):
+        if MarketDataFeed.FX in sources_list:
+            self.download_currency_rates(start_timestamp, end_timestamp)
+        self.download_asset_prices(start_timestamp, end_timestamp, sources_list)
+        logging.info(self.tr("Download completed"))
+
+    # Checks for present quotations of 'asset' in given 'currency' and adjusts 'start' timestamp to be at
+    # the end of available quotes interval if needed.
+    def _adjust_start(self, asset: JalAsset, currency_id: int, start) -> int:
+        quotes_begin, quotes_end = asset.quotes_range(currency_id)
+        if start < quotes_begin:
+            from_timestamp = start
+        else:
+            from_timestamp = quotes_end if quotes_end > start else start
+        return from_timestamp
+
+    def _store_quotations(self, asset: JalAsset, currency_id: int, data: pd.DataFrame) -> None:
+        if data is not None:
+            quotations = []
+            for date, quote in data.iterrows():  # Date in pandas dataset is in UTC by default
+                quotations.append({'timestamp': int(date.timestamp()), 'quote': quote[0]})
+            asset.set_quotes(quotations, currency_id)
+
+    def download_currency_rates(self, start_timestamp, end_timestamp):
+        data_loaders = {
+            "RUB": self.CBR_DataReader,
+            "EUR": self.ECB_DataReader
+        }
+        self.PrepareRussianCBReader()
+        for base in set([x[1] for x in JalAsset.get_base_currency_history(start_timestamp, end_timestamp)]):
+            for currency in JalAsset.get_currencies():
+                if currency.id() == base or currency.quote_source(None) != MarketDataFeed.FX:
+                    continue  # Skip as it is X/X ratio that is always 1
+                from_timestamp = self._adjust_start(currency, base, start_timestamp)
+                if end_timestamp < from_timestamp:
+                    continue
+                try:
+                    data = data_loaders[JalAsset(base).symbol()](currency, from_timestamp, end_timestamp)
+                except (xml_tree.ParseError, pd.errors.EmptyDataError, KeyError):
+                    logging.warning(self.tr("No rates were downloaded for ") +
+                                    f"{currency.symbol()}/{JalAsset(base).symbol()}")
+                    continue
+                self._store_quotations(currency, base, data)
+
+    def download_asset_prices(self, start_timestamp, end_timestamp, sources_list):
+        data_loaders = {
             MarketDataFeed.NA: self.Dummy_DataReader,
-            MarketDataFeed.CBR: self.CBR_DataReader,
             MarketDataFeed.RU: self.MOEX_DataReader,
             MarketDataFeed.EU: self.Euronext_DataReader,
             MarketDataFeed.US: self.Yahoo_Downloader,
@@ -76,46 +125,22 @@ class QuoteDownloader(QObject):
             MarketDataFeed.GB: self.YahooLSE_Downloader,
             MarketDataFeed.FRA: self.YahooFRA_Downloader
         }
-
-    def showQuoteDownloadDialog(self, parent):
-        dialog = QuotesUpdateDialog(parent)
-        if dialog.exec():
-            self.UpdateQuotes(dialog.getStartDate(), dialog.getEndDate(), dialog.getSourceList())
-            self.download_completed.emit()
-
-    def UpdateQuotes(self, start_timestamp, end_timestamp, sources_list):
-        self.PrepareRussianCBReader()
-        assets = JalAsset.get_currencies()
-        # Append base currency id to each currency as currency rate is relative to base currency
-        assets = [{"asset": x, "currency": int(JalSettings().getValue('BaseCurrency'))} for x in assets]
-        assets += JalAsset.get_active_assets(start_timestamp, end_timestamp)  # append assets list
+        assets = JalAsset.get_active_assets(start_timestamp, end_timestamp)  # append assets list
         for asset_data in assets:
             asset = asset_data['asset']
-            if asset.id() == int(JalSettings().getValue('BaseCurrency')):
-                continue
             currency = asset_data['currency']
-            quotes_begin, quotes_end = asset.quotes_range(currency)
-            if start_timestamp < quotes_begin:
-                from_timestamp = start_timestamp
-            else:
-                from_timestamp = quotes_end if quotes_end > start_timestamp else start_timestamp
+            from_timestamp = self._adjust_start(asset, currency, start_timestamp)
             if end_timestamp < from_timestamp:
                 continue
             try:
                 data_source = asset.quote_source(currency)
                 if data_source not in sources_list:   # skip sources that are not requested
                     continue
-                data = self.data_loaders[data_source](asset, currency, from_timestamp, end_timestamp)
+                data = data_loaders[data_source](asset, currency, from_timestamp, end_timestamp)
             except (xml_tree.ParseError, pd.errors.EmptyDataError, KeyError):
-                logging.warning(self.tr("No data were downloaded for ") + f"{asset.name()}")
+                logging.warning(self.tr("No quotes were downloaded for ") + f"{asset.symbol()}")
                 continue
-            if data is not None:
-                quotations = []
-                for date, quote in data.iterrows():  # Date in pandas dataset is in UTC by default
-                    quotations.append({'timestamp': int(date.timestamp()), 'quote': quote[0]})
-                asset.set_quotes(quotations, currency)
-        JalDB().commit()
-        logging.info(self.tr("Download completed"))
+            self._store_quotations(asset, currency, data)
 
     def PrepareRussianCBReader(self):
         rows = []
@@ -133,14 +158,14 @@ class QuoteDownloader(QObject):
     def Dummy_DataReader(self, _asset, _currency_id, _start_timestamp, _end_timestamp):
         return None
 
-    def CBR_DataReader(self, asset, _currency_id, start_timestamp, end_timestamp):
+    def CBR_DataReader(self, currency, start_timestamp, end_timestamp):
         date1 = datetime.utcfromtimestamp(start_timestamp).strftime('%d/%m/%Y')
         # add 1 day to end_timestamp as CBR sets rate are a day ahead
         date2 = (datetime.utcfromtimestamp(end_timestamp) + timedelta(days=1)).strftime('%d/%m/%Y')
         try:
-            code = str(self.CBR_codes.loc[self.CBR_codes["ISO_name"] == asset.symbol(), "CBR_code"].values[0]).strip()
+            code = str(self.CBR_codes.loc[self.CBR_codes["ISO_name"] == currency.symbol(), "CBR_code"].values[0]).strip()
         except IndexError:
-            logging.error(self.tr("Failed to get CBR data for: " + f"{asset.symbol()}"))
+            logging.debug(self.tr("There are no CBR data for: " + f"{currency.symbol()}"))
             return None
         url = f"http://www.cbr.ru/scripts/XML_dynamic.asp?date_req1={date1}&date_req2={date2}&VAL_NM_RQ={code}"
         xml_root = xml_tree.fromstring(get_web_data(url))
@@ -160,6 +185,24 @@ class QuoteDownloader(QObject):
         rates = data.set_index("Date")
         return rates
 
+    def ECB_DataReader(self, currency, start_timestamp, end_timestamp):
+        date1 = datetime.utcfromtimestamp(start_timestamp).strftime('%Y-%m-%d')
+        date2 = datetime.utcfromtimestamp(end_timestamp).strftime('%Y-%m-%d')
+        url = f"https://sdw-wsrest.ecb.europa.eu/service/data/EXR/D.{currency.symbol()}.EUR.SP00.A?startPeriod={date1}&endPeriod={date2}"
+        file = StringIO(get_web_data(url, headers={'Accept': 'text/csv'}))
+        try:
+            data = pd.read_csv(file, dtype={'TIME_PERIOD': str, 'OBS_VALUE': str})
+        except ParserError:
+            return None
+        data.rename(columns={'TIME_PERIOD': 'Date', 'OBS_VALUE': 'Rate'}, inplace=True)
+        data = data[['Date', 'Rate']]  # Keep only required columns
+        data['Date'] = pd.to_datetime(data['Date'], format="%Y-%m-%d")
+        data['Rate'] = data['Rate'].apply(Decimal)   # Convert from str to Decimal
+        data['Rate'] = Decimal('1') / data['Rate']
+        data['Rate'] = data['Rate'].apply(round, args=(10, ))
+        rates = data.set_index("Date")
+        return rates
+
     # Get asset data from http://www.moex.com
     # Accepts parameters:
     #     symbol, isin, reg_number - to identify asset
