@@ -1,12 +1,10 @@
 from PySide6.QtCore import Signal, Slot, Property
 from PySide6.QtWidgets import QApplication, QDialog, QWidget, QPushButton, QComboBox, QMenu, QHBoxLayout, QCheckBox, \
     QMessageBox
-from PySide6.QtSql import QSqlQuery, QSqlTableModel
 from jal.constants import Setup
-from jal.db.db import JalDB
+from jal.db.db import JalModel
 from jal.db.account import JalAccount
 from jal.db.asset import JalAsset
-from jal.db.settings import JalSettings
 from jal.widgets.reference_dialogs import AccountListDialog
 from jal.ui.ui_select_account_dlg import Ui_SelectAccountDlg
 
@@ -98,6 +96,7 @@ class SelectAccountDialog(QDialog, Ui_SelectAccountDlg):
         self.setResult(QDialog.Accepted)
         event.accept()
 
+
 # ----------------------------------------------------------------------------------------------------------------------
 class CurrencyComboBox(QComboBox):
     changed = Signal(int)
@@ -108,11 +107,7 @@ class CurrencyComboBox(QComboBox):
         self.model = None
         self.activated.connect(self.OnUserSelection)
 
-        self.query = QSqlQuery(db=JalDB.connection())
-        self.query.prepare(f"SELECT id, symbol FROM currencies")
-        self.query.exec()
-        self.model = QSqlTableModel(db=JalDB.connection())
-        self.model.setQuery(self.query)
+        self.model = JalModel(self, "currencies")
         self.model.select()
         self.setModel(self.model)
         self.setModelColumn(self.model.fieldIndex("symbol"))
@@ -193,7 +188,7 @@ class OptionalCurrencyComboBox(QWidget):
     def onClick(self):
         if self.null_flag.isChecked():
             if self.currency.selected_id == 0:
-                self.currency.selected_id = JalSettings().getValue('BaseCurrency')
+                self.currency.selected_id = JalAsset.get_base_currency()
             self.currency_id = self.currency.selected_id
         else:
             self.currency_id = 0
