@@ -5,7 +5,7 @@ from PySide6.QtCore import Qt, QModelIndex, QEvent, QLocale, QDateTime, QDate, Q
 from PySide6.QtGui import QDoubleValidator, QBrush, QKeyEvent
 from jal.constants import CustomColor
 from jal.widgets.reference_selector import AssetSelector, PeerSelector, CategorySelector, TagSelector
-from jal.db.db import JalDB
+from jal.db.db import JalModel
 from jal.db.helpers import localize_decimal, delocalize_decimal
 from jal.db.account import JalAccount
 
@@ -67,6 +67,7 @@ class DateTimeEditWithReset(QDateTimeEdit):
 class TimestampDelegate(QStyledItemDelegate):
     def __init__(self, display_format='%d/%m/%Y %H:%M:%S', parent=None):
         QStyledItemDelegate.__init__(self, parent)
+        self._parent = parent
         self._format = display_format
 
     def displayText(self, value, locale):
@@ -95,6 +96,18 @@ class TimestampDelegate(QStyledItemDelegate):
         timestamp = editor.dateTime().toSecsSinceEpoch()
         model.setData(index, timestamp)
 
+    def paint(self, painter, option, index):
+        super().paint(painter, option, index)
+        if type(self._parent) == QTreeView:    # Extra code for tree views - to draw grid lines
+            painter.save()
+            pen = painter.pen()
+            pen.setWidth(1)
+            pen.setStyle(Qt.DotLine)
+            pen.setColor(Qt.GlobalColor.lightGray)
+            painter.setPen(pen)
+            painter.drawRect(option.rect)
+            painter.restore()
+
 
 # -----------------------------------------------------------------------------------------------------------------------
 # Delegate for float numbers formatting
@@ -102,10 +115,12 @@ class TimestampDelegate(QStyledItemDelegate):
 # 'allow_tail' - display more digits for numbers that have more digits than 'tolerance'
 #                and only 'tolerance' digits otherwise
 # 'colors' - make Green/Red background for positive/negative values
+# 'percent' - multiply values by 100 in order to display percents
+# 'empty_zero' - display nothing instead of number 0
 class FloatDelegate(QStyledItemDelegate):
     DEFAULT_TOLERANCE = 6
 
-    def __init__(self, tolerance=None, allow_tail=True, colors=False, percent=False, parent=None):
+    def __init__(self, tolerance=None, allow_tail=True, colors=False, percent=False, empty_zero=False, parent=None):
         self._parent = parent
         QStyledItemDelegate.__init__(self, parent)
         try:
@@ -116,6 +131,7 @@ class FloatDelegate(QStyledItemDelegate):
         self._colors = colors
         self._color = None
         self._percent = percent
+        self._empty_zero = empty_zero
         self._validator = QDoubleValidator()
         self._validator.setLocale(QLocale().system())
 
@@ -130,6 +146,8 @@ class FloatDelegate(QStyledItemDelegate):
             self._color = CustomColor.LightGreen
         elif amount < Decimal('0'):
             self._color = CustomColor.LightRed
+        if amount == Decimal('0') and self._empty_zero:
+            return ''
         decimal_places = -amount.normalize().as_tuple().exponent
         decimal_places = decimal_places if self._allow_tail and (decimal_places > self._tolerance) else self._tolerance
         return QLocale().toString(float(amount), 'f', decimal_places)
@@ -245,36 +263,26 @@ class GridLinesDelegate(QStyledItemDelegate):
 # -----------------------------------------------------------------------------------------------------------------------
 # Base class for lookup delegate that allows Asset, Peer, Category and Tag selection
 class LookupSelectorDelegate(QStyledItemDelegate):
-    Category = 1
-    Tag = 2
-    Peer = 3
-    Asset = 4
-
     def __init__(self, parent=None):
         QStyledItemDelegate.__init__(self, parent)
-        self._type = 0
         self._table = ''
         self._field = ''
+        self._selector = None
 
     def displayText(self, value, locale):
-        item_name = JalDB.get_db_value(self._table, self._field, "id", value)
+        item_name = JalModel(self, self._table).get_value(self._field, "id", value)
         if item_name is None:
             return ''
         else:
             return item_name
 
+    def createSelector(self, parent) -> None:
+        raise NotImplementedError("Method createSelector() isn't defined")
+
     def createEditor(self, aParent, option, index):
-        if self._type == self.Category:
-            selector = CategorySelector(aParent)
-        elif self._type == self.Tag:
-            selector = TagSelector(aParent)
-        elif self._type == self.Peer:
-            selector = PeerSelector(aParent)
-        elif self._type == self.Asset:
-            selector = AssetSelector(aParent)
-        else:
-            raise ValueError
-        return selector
+        self.createSelector(aParent)
+        assert self._selector is not None, "Selector isn't created in LookupSelectorDelegate descendant"
+        return self._selector
 
     def setEditorData(self, editor: QWidget, index: QModelIndex) -> None:
         editor.selected_id = index.data()
@@ -286,30 +294,38 @@ class LookupSelectorDelegate(QStyledItemDelegate):
 class CategorySelectorDelegate(LookupSelectorDelegate):
     def __init__(self, parent=None):
         LookupSelectorDelegate.__init__(self, parent)
-        self._type = LookupSelectorDelegate.Category
         self._table = "categories"
         self._field = "name"
 
+    def createSelector(self, parent) -> None:
+        self._selector = CategorySelector(parent)
+
 
 class TagSelectorDelegate(LookupSelectorDelegate):
     def __init__(self, parent=None):
         LookupSelectorDelegate.__init__(self, parent)
-        self._type = LookupSelectorDelegate.Tag
         self._table = "tags"
         self._field = "tag"
 
+    def createSelector(self, parent) -> None:
+        self._selector = TagSelector(parent)
+
 
 class PeerSelectorDelegate(LookupSelectorDelegate):
     def __init__(self, parent=None):
         LookupSelectorDelegate.__init__(self, parent)
-        self._type = LookupSelectorDelegate.Peer
         self._table = "agents"
         self._field = "name"
 
+    def createSelector(self, parent) -> None:
+        self._selector = PeerSelector(parent)
+
 
 class AssetSelectorDelegate(LookupSelectorDelegate):
     def __init__(self, parent=None):
         LookupSelectorDelegate.__init__(self, parent)
-        self._type = LookupSelectorDelegate.Asset
         self._table = "assets_ext"
         self._field = "symbol"
+
+    def createSelector(self, parent) -> None:
+        self._selector = AssetSelector(parent)
