@@ -8,7 +8,7 @@ from functools import partial
 
 from PySide6.QtCore import Qt, Slot, QDir, QLocale, QMetaObject
 from PySide6.QtGui import QIcon, QActionGroup, QAction
-from PySide6.QtWidgets import QApplication, QMainWindow, QMessageBox, QProgressBar
+from PySide6.QtWidgets import QApplication, QMainWindow, QMessageBox, QProgressBar, QMenu
 
 from jal import __version__
 from jal.ui.ui_main_window import Ui_JAL_MainWindow
@@ -16,7 +16,7 @@ from jal.widgets.operations_widget import OperationsWidget
 from jal.widgets.tax_widget import TaxWidget, MoneyFlowWidget
 from jal.widgets.helpers import dependency_present
 from jal.widgets.reference_dialogs import AccountListDialog, AssetListDialog, TagsListDialog,\
-    CategoryListDialog, CountryListDialog, QuotesListDialog, PeerListDialog
+    CategoryListDialog, QuotesListDialog, PeerListDialog, BaseCurrencyDialog
 from jal.constants import Setup
 from jal.db.backup_restore import JalBackup
 from jal.db.helpers import get_app_path, get_dbfilename, load_icon
@@ -98,10 +98,10 @@ class MainWindow(QMainWindow, Ui_JAL_MainWindow):
         self.actionPeers.triggered.connect(partial(self.onDataDialog, "agents"))
         self.actionCategories.triggered.connect(partial(self.onDataDialog, "categories"))
         self.actionTags.triggered.connect(partial(self.onDataDialog, "tags"))
-        self.actionCountries.triggered.connect(partial(self.onDataDialog, "countries"))
         self.actionQuotes.triggered.connect(partial(self.onDataDialog, "quotes"))
-        self.PrepareTaxForms.triggered.connect(partial(self.mdiArea.addSubWindow, TaxWidget()))
-        self.PrepareFlowReport.triggered.connect(partial(self.mdiArea.addSubWindow, MoneyFlowWidget()))
+        self.actionBaseCurrency.triggered.connect(partial(self.onDataDialog, "base_currency"))
+        self.PrepareTaxForms.triggered.connect(partial(TaxWidget.showInMDI, self.mdiArea))
+        self.PrepareFlowReport.triggered.connect(partial(MoneyFlowWidget.showInMDI, self.mdiArea))
         self.downloader.download_completed.connect(self.updateWidgets)
         self.ledger.updated.connect(self.updateWidgets)
         self.statements.load_completed.connect(self.onStatementImport)
@@ -112,7 +112,7 @@ class MainWindow(QMainWindow, Ui_JAL_MainWindow):
         if self.running:
             return
         self.running = True
-        # Call slot via queued connection so it's called from the UI thread after the window has been shown
+        # Call slot via queued connection, so it's called from the UI thread after the window has been shown
         QMetaObject().invokeMethod(self, "afterShowEvent", Qt.ConnectionType.QueuedConnection)
 
     @Slot()
@@ -182,10 +182,18 @@ class MainWindow(QMainWindow, Ui_JAL_MainWindow):
 
     # Create menu entry for all known reports based on self.reports.sources values
     def createReportsMenu(self):
+        groups = {}
         for i, report in enumerate(self.reports.items):
             action = QAction(report['name'].replace('&', '&&'), self)  # & -> && to prevent shortcut creation
             action.setData(i)
-            self.menuReports.addAction(action)
+            if report['group']:
+                if report['group'] not in groups:
+                    groups[report['group']] = QMenu(report['group'], self.menuReports)
+                    self.menuReports.addAction(groups[report['group']].menuAction())
+                submenu = groups[report['group']]
+                submenu.addAction(action)
+            else:
+                self.menuReports.addAction(action)
             self.reportsGroup.addAction(action)
 
     @Slot()
@@ -198,14 +206,15 @@ class MainWindow(QMainWindow, Ui_JAL_MainWindow):
         about_box = QMessageBox(self)
         about_box.setAttribute(Qt.WA_DeleteOnClose)
         about_box.setWindowTitle(self.tr("About"))
-        title = self.tr("<h3>JAL</h3><p>Just Another Ledger, version {version}</p>".format(version=__version__))
+        version = f"{__version__} (db{Setup.DB_REQUIRED_VERSION})"
+        title = "<h3>JAL</h3><p>Just Another Ledger, " + self.tr("version") + " " + version +"</p>"
         about_box.setText(title)
-        about = self.tr("<p>More information, manuals and problem reports are at "
-                        "<a href=https://github.com/titov-vv/jal>github home page</a></p>"
-                        "<p>Questions, comments, help or donations:</p>"
-                        "<p><a href=mailto:jal@gmx.ru>jal@gmx.ru</a></p>"
-                        "<p><a href=https://t.me/jal_support>Telegram</a></p>")
-        about_box.setInformativeText(about)
+        about_text = "<p>" + self.tr("More information, manuals and problem reports are at ") + \
+                     "<a href=https://github.com/titov-vv/jal>" + self.tr("github home page") + "</a></p><p>" + \
+                     self.tr("Questions, comments, help or donations:") + \
+                     "</p><p><a href=mailto:jal@gmx.ru>jal@gmx.ru</a></p>" + \
+                     "<p><a href=https://t.me/jal_support>Telegram</a></p>"
+        about_box.setInformativeText(about_text)
         about_box.show()
 
     def showProgressBar(self, visible=False):
@@ -230,17 +239,18 @@ class MainWindow(QMainWindow, Ui_JAL_MainWindow):
         elif dlg_type == "assets":
             AssetListDialog().exec()
         elif dlg_type == "agents":
-            PeerListDialog().exec()
+            PeerListDialog(self).exec()
         elif dlg_type == "categories":
-            CategoryListDialog().exec()
+            CategoryListDialog(self).exec()
         elif dlg_type == "tags":
-            TagsListDialog().exec()
-        elif dlg_type == "countries":
-            CountryListDialog().exec()
+            TagsListDialog(self).exec()
         elif dlg_type == "quotes":
             QuotesListDialog().exec()
+        elif dlg_type == "base_currency":
+            BaseCurrencyDialog().exec()
         else:
             assert False, f"Unexpected dialog call: '{dlg_type}'"
+        self.ledger.rebuild()
 
     @Slot()
     def updateWidgets(self):
