@@ -1,13 +1,13 @@
 from functools import partial
 
-from PySide6.QtCore import Qt, Slot, Signal, QDateTime
+from PySide6.QtCore import Qt, Slot, Signal, QDateTime, QSortFilterProxyModel
 from PySide6.QtGui import QAction
 from PySide6.QtWidgets import QMenu, QMessageBox
 from jal.ui.ui_operations_widget import Ui_OperationsWidget
 from jal.widgets.mdi import MdiWidget
 from jal.db.helpers import load_icon
 from jal.db.account import JalAccount
-from jal.db.settings import JalSettings
+from jal.db.asset import JalAsset
 from jal.db.balances_model import BalancesModel
 from jal.db.operations_model import OperationsModel
 from jal.db.operations import LedgerTransaction
@@ -44,7 +44,10 @@ class OperationsWidget(MdiWidget, Ui_OperationsWidget):
         self.balances_model.configureView()
 
         self.operations_model = OperationsModel(self.OperationsTableView)
-        self.OperationsTableView.setModel(self.operations_model)
+        self.operations_filtered_model = QSortFilterProxyModel(self.OperationsTableView)
+        self.operations_filtered_model.setSourceModel(self.operations_model)
+        self.operations_filtered_model.setFilterCaseSensitivity(Qt.CaseInsensitive)
+        self.OperationsTableView.setModel(self.operations_filtered_model)
         self.operations_model.configureView()
         self.OperationsTableView.setContextMenuPolicy(Qt.CustomContextMenu)
 
@@ -61,7 +64,7 @@ class OperationsWidget(MdiWidget, Ui_OperationsWidget):
         current_time = QDateTime.currentDateTime()
         current_time.setTimeSpec(Qt.UTC)  # We use UTC everywhere so need to force TZ info
         self.BalanceDate.setDateTime(current_time)
-        self.BalancesCurrencyCombo.setIndex(JalSettings().getValue('BaseCurrency'))
+        self.BalancesCurrencyCombo.setIndex(JalAsset.get_base_currency())
 
         self.OperationsTableView.selectRow(0)
         self.DateRange.setCurrentIndex(0)
@@ -73,7 +76,7 @@ class OperationsWidget(MdiWidget, Ui_OperationsWidget):
         self.BalancesTableView.doubleClicked.connect(self.OnBalanceDoubleClick)
         self.ShowInactiveCheckBox.stateChanged.connect(self.BalancesTableView.model().toggleActive)
         self.DateRange.changed.connect(self.operations_model.setDateRange)
-        self.ChooseAccountBtn.changed.connect(self.OperationsTableView.model().setAccount)
+        self.ChooseAccountBtn.changed.connect(self.operations_model.setAccount)
         self.SearchString.editingFinished.connect(self.updateOperationsFilter)
         self.OperationsTableView.selectionModel().selectionChanged.connect(self.OnOperationChange)
         self.OperationsTableView.customContextMenuRequested.connect(self.onOperationContextMenu)
@@ -100,7 +103,8 @@ class OperationsWidget(MdiWidget, Ui_OperationsWidget):
 
     @Slot()
     def updateOperationsFilter(self):
-        self.OperationsTableView.model().filterText(self.SearchString.text())
+        self.OperationsTableView.model().setFilterFixedString(self.SearchString.text())
+        self.OperationsTableView.model().setFilterKeyColumn(-1)
 
     @Slot()
     def OnBalanceDoubleClick(self, index):
@@ -113,8 +117,8 @@ class OperationsWidget(MdiWidget, Ui_OperationsWidget):
         if len(self.OperationsTableView.selectionModel().selectedRows()) == 1:
             idx = selected.indexes()
             if idx:
-                selected_row = idx[0].row()
-                op_type, op_id = self.OperationsTableView.model().get_operation(selected_row)
+                selected_row = self.operations_filtered_model.mapToSource(idx[0]).row()
+                op_type, op_id = self.operations_model.get_operation(selected_row)
         self.OperationsTabs.show_operation(op_type, op_id)
 
     @Slot()
