@@ -1,14 +1,19 @@
-from PySide6.QtCore import Qt, QModelIndex
+import logging
+from PySide6.QtCore import Qt, Slot, QDate
+from PySide6.QtGui import QAction
 from PySide6.QtSql import QSqlRelation, QSqlRelationalDelegate, QSqlIndex
-from PySide6.QtWidgets import QAbstractItemView
+from PySide6.QtWidgets import QAbstractItemView, QMenu, QDialog, QMessageBox
 from jal.constants import PredefindedAccountType, PredefinedAsset
-from jal.db.db import JalDB
+from jal.db.peer import JalPeer
+from jal.db.category import JalCategory
+from jal.db.tag import JalTag
 from jal.db.reference_models import AbstractReferenceListModel, SqlTreeModel
-from jal.widgets.delegates import TimestampDelegate, BoolDelegate, FloatDelegate, \
-    PeerSelectorDelegate, AssetSelectorDelegate
+from jal.widgets.delegates import TimestampDelegate, BoolDelegate, FloatDelegate, PeerSelectorDelegate, \
+    AssetSelectorDelegate
 from jal.widgets.reference_data import ReferenceDataDialog
 from jal.widgets.asset_dialog import AssetDialog
 from jal.widgets.delegates import GridLinesDelegate
+from jal.widgets.selection_dialog import SelectPeerDialog, SelectCategoryDialog, SelectTagDialog 
 
 
 # ----------------------------------------------------------------------------------------------------------------------
@@ -33,7 +38,7 @@ class AccountListModel(AbstractReferenceListModel):
         self._peer_delegate = None
         self._timestamp_delegate = None
         self._bool_delegate = None
-        self._default_values = {'active': 1, 'reconciled_on': 0, 'country_id': 0}
+        self._default_values = {'active': 1, 'reconciled_on': 0, 'country_id': 0, 'precision': 2}
         self.setRelation(self.fieldIndex("currency_id"), QSqlRelation("currencies", "id", "symbol"))
         self.setRelation(self.fieldIndex("country_id"), QSqlRelation("countries", "id", "code"))
 
@@ -54,19 +59,6 @@ class AccountListModel(AbstractReferenceListModel):
         self._bool_delegate = BoolDelegate(self._view)
         self._view.setItemDelegateForColumn(self.fieldIndex("active"), self._bool_delegate)
 
-    def getAccountType(self, item_id: int) -> int:
-        type_id = JalDB.readSQL(f"SELECT type_id FROM {self._table} WHERE id=:id", [(":id", item_id)])
-        type_id = 0 if type_id is None else type_id
-        return type_id
-
-    def locateItem(self, item_id, use_filter=''):
-        row = JalDB.readSQL(f"SELECT row_number FROM ("
-                             f"SELECT ROW_NUMBER() OVER (ORDER BY {self._default_name}) AS row_number, id "
-                             f"FROM {self._table} WHERE {use_filter}) WHERE id=:id", [(":id", item_id)])
-        if row is None:
-            return QModelIndex()
-        return self.index(row-1, 0)
-
 
 class AccountListDialog(ReferenceDataDialog):
     def __init__(self):
@@ -94,7 +86,7 @@ class AccountListDialog(ReferenceDataDialog):
         self.group_id = 1
 
     def locateItem(self, item_id):
-        type_id = self.model.getAccountType(item_id)
+        type_id = self.model.getGroupId(item_id)
         if type_id == 0:
             return
         self.GroupCombo.setCurrentIndex(type_id-1)
@@ -126,7 +118,7 @@ class AssetListModel(AbstractReferenceListModel):
         self._timestamp_delegate = None
         self._default_values = {'isin': '', 'country_id': 0, 'quote_source': -1}
         self.setRelation(self.fieldIndex("currency_id"), QSqlRelation("currencies", "id", "symbol"))
-        self.setRelation(self.fieldIndex("country_id"), QSqlRelation("countries", "id", "name"))
+        self.setRelation(self.fieldIndex("country_id"), QSqlRelation("countries_ext", "id", "name"))
         self.setRelation(self.fieldIndex("quote_source"), QSqlRelation("data_sources", "id", "name"))
 
     def configureView(self):
@@ -135,19 +127,6 @@ class AssetListModel(AbstractReferenceListModel):
         self._view.setItemDelegateForColumn(self.fieldIndex("country_id"), self._lookup_delegate)
         self._view.setItemDelegateForColumn(self.fieldIndex("quote_source"), self._lookup_delegate)
 
-    def getAssetType(self, item_id: int) -> int:
-        type_id = JalDB.readSQL(f"SELECT type_id FROM {self._table} WHERE id=:id", [(":id", item_id)])
-        type_id = 0 if type_id is None else type_id
-        return type_id
-
-    def locateItem(self, item_id, use_filter=''):
-        row = JalDB.readSQL(f"SELECT row_number FROM ("
-                             f"SELECT ROW_NUMBER() OVER (ORDER BY {self._default_name}) AS row_number, id "
-                             f"FROM {self._table} WHERE {use_filter}) WHERE id=:id", [(":id", item_id)])
-        if row is None:
-            return QModelIndex()
-        return self.index(row-1, 0)
-
 
 class AssetListDialog(ReferenceDataDialog):
     def __init__(self):
@@ -183,7 +162,7 @@ class AssetListDialog(ReferenceDataDialog):
         # action.triggered.connect(self.updateExchangeData)
 
     def locateItem(self, item_id):
-        type_id = self.model.getAssetType(item_id)
+        type_id = self.model.getGroupId(item_id)
         if type_id == 0:
             return
         self.GroupCombo.setCurrentIndex(type_id-1)
@@ -210,9 +189,8 @@ class PeerTreeModel(SqlTreeModel):
             return None
         item_id = index.internalId()
         if role == Qt.DisplayRole:
-            if index.column() == 2:
-                return JalDB.readSQL("SELECT COUNT(d.id) FROM agents AS p "
-                                      "LEFT JOIN actions AS d ON d.peer_id=p.id WHERE p.id=:id", [(":id", item_id)])
+            if index.column() == self.fieldIndex("actions_count"):
+                return JalPeer(item_id).number_of_documents()
             else:
                 return super().data(index, role)
         return None
@@ -227,14 +205,20 @@ class PeerTreeModel(SqlTreeModel):
 
 
 class PeerListDialog(ReferenceDataDialog):
-    def __init__(self):
-        ReferenceDataDialog.__init__(self)
+    def __init__(self, parent):
+        ReferenceDataDialog.__init__(self, parent)
         self.table = "agents"
         self.model = PeerTreeModel(self.table, self.TreeView)
         self.TreeView.setModel(self.model)
         self.model.configureView()
         self.setup_ui()
         super()._init_completed()
+        self._menu_peer_id = 0
+        self._menu_peer_name = ''
+        self.actionShowUsage = QAction(text=self.tr("Show operations with Peer"), parent=self)
+        self.actionReplace = QAction(text=self.tr("Replace with..."), parent=self)
+        self.actionShowUsage.triggered.connect(self.showUsageReport)
+        self.actionReplace.triggered.connect(self.replacePeer)
 
     def setup_ui(self):
         self.search_field = "name"
@@ -243,9 +227,36 @@ class PeerListDialog(ReferenceDataDialog):
         self.SearchFrame.setVisible(True)
         self.setWindowTitle(self.tr("Peers"))
         self.Toggle.setVisible(False)
+        if hasattr(self._parent, "reports"):  # Activate menu only if dialog is called from main window menu
+            self.custom_context_menu = True
 
     def locateItem(self, item_id):
         self.model.locateItem(item_id)
+        
+    def customizeContextMenu(self, menu: QMenu, index):
+        self._menu_peer_id = self.model.getId(index)
+        self._menu_peer_name = self.model.getName(index)
+        menu.addAction(self.actionShowUsage)
+        menu.addAction(self.actionReplace)
+
+    @Slot()
+    def showUsageReport(self):
+        settings = {'begin_ts': 0, 'end_ts': QDate.currentDate().endOfDay(Qt.UTC).toSecsSinceEpoch(),
+                    'peer_id': self._menu_peer_id}
+        self._parent.reports.show_report("PeerReportWindow", settings, maximized=True)
+
+    @Slot()
+    def replacePeer(self):
+        dialog = SelectPeerDialog(self.tr("Replace peer '") + self._menu_peer_name + self.tr("' with: "))
+        if dialog.exec() != QDialog.Accepted:
+            return
+        reply = QMessageBox().warning(self, '', self.tr("Keep old name in notes?"), QMessageBox.Yes, QMessageBox.No)
+        if reply == QMessageBox.Yes:
+            JalPeer(self._menu_peer_id).replace_with(dialog.selected_id, old_name=self._menu_peer_name)
+        else:
+            JalPeer(self._menu_peer_id).replace_with(dialog.selected_id)
+        logging.info(self.tr("Peer '") + self._menu_peer_name + self.tr("' was successfully replaced"))
+        self.close()
 
 
 # ----------------------------------------------------------------------------------------------------------------------
@@ -267,14 +278,20 @@ class CategoryTreeModel(SqlTreeModel):
 
 
 class CategoryListDialog(ReferenceDataDialog):
-    def __init__(self):
-        ReferenceDataDialog.__init__(self)
+    def __init__(self, parent):
+        ReferenceDataDialog.__init__(self, parent)
         self.table = "categories"
         self.model = CategoryTreeModel(self.table, self.TreeView)
         self.TreeView.setModel(self.model)
         self.model.configureView()
         self.setup_ui()
         super()._init_completed()
+        self._menu_category_id = 0
+        self._menu_category_name = ''
+        self.actionShowUsage = QAction(text=self.tr("Show operations with Category"), parent=self)
+        self.actionReplace = QAction(text=self.tr("Replace with..."), parent=self)
+        self.actionShowUsage.triggered.connect(self.showUsageReport)
+        self.actionReplace.triggered.connect(self.replaceCategory)
 
     def setup_ui(self):
         self.search_field = "name"
@@ -283,10 +300,33 @@ class CategoryListDialog(ReferenceDataDialog):
         self.SearchFrame.setVisible(True)
         self.setWindowTitle(self.tr("Categories"))
         self.Toggle.setVisible(False)
+        if hasattr(self._parent, "reports"):  # Activate menu only if dialog is called from main window menu
+            self.custom_context_menu = True
 
     def locateItem(self, item_id):
         self.model.locateItem(item_id)
 
+    def customizeContextMenu(self, menu: QMenu, index):
+        self._menu_category_id = self.model.getId(index)
+        self._menu_category_name = self.model.getName(index)
+        menu.addAction(self.actionShowUsage)
+        menu.addAction(self.actionReplace)
+
+    @Slot()
+    def showUsageReport(self):
+        settings = {'begin_ts': 0, 'end_ts': QDate.currentDate().endOfDay(Qt.UTC).toSecsSinceEpoch(),
+                    'category_id': self._menu_category_id}
+        self._parent.reports.show_report("CategoryReportWindow", settings, maximized=True)
+
+    @Slot()
+    def replaceCategory(self):
+        dialog = SelectCategoryDialog(self.tr("Replace category '") + self._menu_category_name + self.tr("' with: "))
+        if dialog.exec() != QDialog.Accepted:
+            return
+        JalCategory(self._menu_category_id).replace_with(dialog.selected_id)
+        logging.info(self.tr("Category '") + self._menu_category_name + self.tr("' was successfully replaced"))
+        self.close()
+
 # ----------------------------------------------------------------------------------------------------------------------
 class TagListModel(AbstractReferenceListModel):
     def __init__(self, table, parent_view):
@@ -298,73 +338,55 @@ class TagListModel(AbstractReferenceListModel):
         self._hidden = ["id"]
         self._stretch = "tag"
 
-    def locateItem(self, item_id, use_filter=''):
-        row = JalDB.readSQL(f"SELECT row_number FROM (SELECT ROW_NUMBER() OVER (ORDER BY tag) AS row_number, id "
-                             f"FROM {self._table}) WHERE id=:id", [(":id", item_id)])
-        if row is None:
-            return QModelIndex()
-        return self.index(row - 1, 0)
-
 
 class TagsListDialog(ReferenceDataDialog):
-    def __init__(self):
-        ReferenceDataDialog.__init__(self)
+    def __init__(self, parent):
+        ReferenceDataDialog.__init__(self, parent)
         self.table = "tags"
         self.model = TagListModel(self.table, self.DataView)
         self.DataView.setModel(self.model)
         self.model.configureView()
         self.setup_ui()
         super()._init_completed()
+        self._menu_tag_id = 0
+        self._menu_tag_name = ''
+        self.actionShowUsage = QAction(text=self.tr("Show operations with Tag"), parent=self)
+        self.actionReplace = QAction(text=self.tr("Replace with..."), parent=self)
+        self.actionShowUsage.triggered.connect(self.showUsageReport)
+        self.actionReplace.triggered.connect(self.replaceTag)
 
     def setup_ui(self):
         self.search_field = "tag"
         self.setWindowTitle(self.tr("Tags"))
         self.SearchFrame.setVisible(True)
         self.Toggle.setVisible(False)
+        if hasattr(self._parent, "reports"):  # Activate menu only if dialog is called from main window menu
+            self.custom_context_menu = True
 
     def locateItem(self, item_id):
         item_idx = self.model.locateItem(item_id)
         self.DataView.setCurrentIndex(item_idx)
 
+    def customizeContextMenu(self, menu: QMenu, index):
+        self._menu_tag_id = self.model.getId(index)
+        self._menu_tag_name = self.model.getName(index)
+        menu.addAction(self.actionShowUsage)
+        menu.addAction(self.actionReplace)
 
-# ----------------------------------------------------------------------------------------------------------------------
-class CountryListModel(AbstractReferenceListModel):
-    def __init__(self, table, parent_view):
-        AbstractReferenceListModel.__init__(self, table, parent_view)
-        self._columns = [("id", ''),
-                         ("name", self.tr("Country")),
-                         ("code", self.tr("Code")),
-                         ("iso_code", self.tr("ISO code")),
-                         ("tax_treaty", self.tr("Tax Treaty"))]
-        self._sort_by = "name"
-        self._hidden = ["id"]
-        self._stretch = "name"
-        self._default_values = {'tax_treaty': 0}
-        self._bool_delegate = None
+    @Slot()
+    def showUsageReport(self):
+        settings = {'begin_ts': 0, 'end_ts': QDate.currentDate().endOfDay(Qt.UTC).toSecsSinceEpoch(),
+                    'tag_id': self._menu_tag_id}
+        self._parent.reports.show_report("TagReportWindow", settings, maximized=True)
 
-    def configureView(self):
-        super().configureView()
-        self._view.setColumnWidth(self.fieldIndex("code"), 50)
-
-        self._bool_delegate = BoolDelegate(self._view)
-        self._view.setItemDelegateForColumn(self.fieldIndex("tax_treaty"), self._bool_delegate)
-
-
-class CountryListDialog(ReferenceDataDialog):
-    def __init__(self):
-        ReferenceDataDialog.__init__(self)
-        self.table = "countries"
-        self.model = CountryListModel(self.table, self.DataView)
-        self.DataView.setModel(self.model)
-        self.model.configureView()
-        self.setup_ui()
-        super()._init_completed()
-
-    def setup_ui(self):
-        self.search_field = "name"
-        self.SearchFrame.setVisible(True)
-        self.setWindowTitle(self.tr("Countries"))
-        self.Toggle.setVisible(False)
+    @Slot()
+    def replaceTag(self):
+        dialog = SelectTagDialog(self.tr("Replace tag '") + self._menu_tag_name + self.tr("' with: "))
+        if dialog.exec() != QDialog.Accepted:
+            return
+        JalTag(self._menu_tag_id).replace_with(dialog.selected_id)
+        logging.info(self.tr("Tag '") + self._menu_tag_name + self.tr("' was successfully replaced"))
+        self.close()
 
 
 # ----------------------------------------------------------------------------------------------------------------------
@@ -415,4 +437,45 @@ class QuotesListDialog(ReferenceDataDialog):
         self.setWindowTitle(self.tr("Quotes"))
         self.Toggle.setVisible(False)
 
+
+# ----------------------------------------------------------------------------------------------------------------------
+class BaseCurrencyListModel(AbstractReferenceListModel):
+    def __init__(self, table, parent_view):
+        AbstractReferenceListModel.__init__(self, table, parent_view)
+        self._columns = [("id", ''),
+                         ("since_timestamp", self.tr("Date")),
+                         ("currency_id", self.tr("Currency"))]
+        self._hidden = ["id"]
+        self._sort_by = "since_timestamp"
+        self._default_name = "currency_id"
+        self._timestamp_delegate = None
+        self._lookup_delegate = None
+        self.setRelation(self.fieldIndex("currency_id"), QSqlRelation("currencies", "id", "symbol"))
+
+    def configureView(self):
+        super().configureView()
+        self._view.setColumnWidth(self.fieldIndex("since_timestamp"),
+                                  self._view.fontMetrics().horizontalAdvance("00/00/0000") * 1.1)
+        self._view.setColumnWidth(self.fieldIndex("currency_id"), 100)
+
+        self._timestamp_delegate = TimestampDelegate(display_format='%d/%m/%Y', parent=self._view)
+        self._view.setItemDelegateForColumn(self.fieldIndex("since_timestamp"), self._timestamp_delegate)
+        self._lookup_delegate = QSqlRelationalDelegate(self._view)
+        self._view.setItemDelegateForColumn(self.fieldIndex("currency_id"), self._lookup_delegate)
+
+
+class BaseCurrencyDialog(ReferenceDataDialog):
+    def __init__(self):
+        ReferenceDataDialog.__init__(self)
+        self.table = "base_currency"
+        self.model = BaseCurrencyListModel(self.table, self.DataView)
+        self.DataView.setModel(self.model)
+        self.model.configureView()
+        self.setup_ui()
+        super()._init_completed()
+
+    def setup_ui(self):
+        self.setWindowTitle(self.tr("Base currency"))
+        self.Toggle.setVisible(False)
+
 # ----------------------------------------------------------------------------------------------------------------------
