@@ -4,15 +4,14 @@ import logging
 import traceback
 
 from PySide6.QtCore import Property, Slot
-from PySide6.QtWidgets import QFileDialog, QMessageBox
+from PySide6.QtWidgets import QFileDialog, QMessageBox, QApplication
 from jal.ui.ui_tax_export_widget import Ui_TaxWidget
 from jal.ui.ui_flow_export_widget import Ui_MoneyFlowWidget
 from jal.widgets.mdi import MdiWidget
 from jal.widgets.helpers import ts2d
 from jal.db.asset import JalAsset
 from jal.db.peer import JalPeer
-from jal.db.country import JalCountry
-from jal.data_export.taxes import TaxesRus
+from jal.data_export.taxes import TaxReport
 from jal.data_export.taxes_flow import TaxesFlowRus
 from jal.data_export.xlsx import XLSX
 from jal.data_export.dlsg import DLSG
@@ -23,10 +22,34 @@ class TaxWidget(MdiWidget, Ui_TaxWidget):
         MdiWidget.__init__(self)
         self.setupUi(self)
 
+        self.Country.clear()
+        self.Country.addItems([TaxReport.countries[x]['name'] for x in TaxReport.countries])
+        self.Country.currentIndexChanged.connect(self.OnCountryChange)
         self.Year.setValue(datetime.now().year - 1)   # Set previous year by default
         self.XlsSelectBtn.pressed.connect(partial(self.OnFileBtn, 'XLS'))
         self.DlsgSelectBtn.pressed.connect(partial(self.OnFileBtn, 'DLSG'))
         self.SaveButton.pressed.connect(self.SaveReport)
+        self.Country.setCurrentIndex(TaxReport.RUSSIA)
+
+    def OnCountryChange(self, item_id):
+        if item_id == TaxReport.PORTUGAL:
+            self.RussianSpecificFrame.setVisible(False)
+        elif item_id == TaxReport.RUSSIA:
+            self.RussianSpecificFrame.setVisible(True)
+        else:
+            raise ValueError("Selected item has no country handler in code")
+        # Refresh and adjust MDI-window size
+        if not self.parent() is None:
+            self.parent().update()
+            QApplication.processEvents()
+            if not self.parent().isMaximized():  # Prevent size-change of maximized MDI
+                self.parent().adjustSize()
+
+    # Displays tax widget in a given MDI area.
+    # It is implemented as a separate static method in order to prevent unexpected object deletion
+    @staticmethod
+    def showInMDI(parent_mdi):
+        parent_mdi.addSubWindow(TaxWidget(), maximized=False)
 
     @Slot()
     def OnFileBtn(self, type):
@@ -84,33 +107,25 @@ class TaxWidget(MdiWidget, Ui_TaxWidget):
             QMessageBox().warning(self, self.tr("Data are incomplete"),
                                   self.tr("You haven't selected an account for tax report"), QMessageBox.Ok)
             return
-        taxes = TaxesRus()
+        taxes = TaxReport.create_report(self.Country.currentIndex())
+
         tax_report = taxes.prepare_tax_report(self.year, self.account, use_settlement=(not self.no_settelement))
+        if not tax_report:
+            logging.warning(self.tr("Tax report is empty"))
+            return
 
         reports_xls = XLSX(self.xls_filename)
-        templates = {
-            "Дивиденды": "tax_rus_dividends.json",
-            "Акции": "tax_rus_trades.json",
-            "Облигации": "tax_rus_bonds.json",
-            "ПФИ": "tax_rus_derivatives.json",
-            "Криптовалюты": "tax_rus_crypto.json",
-            "Корп.события": "tax_rus_corporate_actions.json",
-            "Комиссии": "tax_rus_fees.json",
-            "Проценты": "tax_rus_interests.json"
-        }
         parameters = {
             "period": f"{ts2d(taxes.year_begin)} - {ts2d(taxes.year_end - 1)}",
-            "account": f"{taxes.account.number()} ({taxes.account.currency()})",
+            "account": f"{taxes.account.number()} ({JalAsset(taxes.account.currency()).symbol()})",
             "currency": JalAsset(taxes.account.currency()).symbol(),
             "broker_name": JalPeer(taxes.account.organization()).name(),
-            "broker_iso_country": JalCountry(taxes.account.country()).iso_code()
+            "broker_iso_country": taxes.account.country().iso_code()
         }
         for section in tax_report:
-            if section not in templates:
-                continue
-            reports_xls.output_data(tax_report[section], templates[section], parameters)
+            reports_xls.output_data(tax_report[section], taxes.report_template(section), parameters)
         reports_xls.save()
-        logging.info(self.tr("Tax report saved to file ") + f"'{self.xls_filename}'")
+        logging.info(self.tr("Tax report was saved to file ") + f"'{self.xls_filename}'")
 
         if self.update_dlsg:
             tax_forms = DLSG(self.year, broker_as_income=self.dlsg_broker_as_income,
@@ -133,6 +148,12 @@ class MoneyFlowWidget(MdiWidget, Ui_MoneyFlowWidget):
         self.XlsSelectBtn.pressed.connect(self.OnFileBtn)
         self.SaveButton.pressed.connect(self.SaveReport)
 
+    # Displays tax widget in a given MDI area.
+    # It is implemented as a separate static method in order to prevent unexpected object deletion
+    @staticmethod
+    def showInMDI(parent_mdi):
+        parent_mdi.addSubWindow(MoneyFlowWidget(), maximized=False)
+
     @Slot()
     def OnFileBtn(self):
         selector = (self.tr("Save money flow report to:"), self.tr("Excel files (*.xlsx)"), '.xlsx', self.XlsFileName)
